<?php
if ( ! class_exists( 'bookingpress_class_paypal' ) ) {
	class bookingpress_class_paypal {
		var $bookingpress_payment_mode    = '';
		var $bookingpress_is_sandbox_mode = true;
		var $bookingpress_api_username    = '';
		var $bookingpress_api_password    = '';
		var $bookingpress_api_signature   = '';
		var $bookingpress_merchant_email  = '';
		var $bookingpress_gateway_status  = '';

		var $bookingpress_paypal = false;

		function __construct() {
			add_filter( 'bookingpress_paypal_submit_form_data', array( $this, 'bookingpress_submit_form_data' ), 10, 2 );
			add_action( 'wp', array( $this, 'bookingpress_payment_gateway_data' ) );
		}

		function arm_init_paypal() {
			$PayPal = false;
			if ( file_exists( BOOKINGPRESS_DIR . '/lib/paypal/paypal.class.php' ) ) {
				require_once BOOKINGPRESS_DIR . '/lib/paypal/paypal.class.php';

				global $wpdb, $BookingPress;

				$this->bookingpress_payment_mode    = $BookingPress->bookingpress_get_settings( 'paypal_payment_mode', 'payment_setting' );
				$this->bookingpress_is_sandbox_mode = ( $this->bookingpress_payment_mode != 'live' ) ? true : false;
				$this->bookingpress_gateway_status  = $BookingPress->bookingpress_get_settings( 'paypal_payment', 'payment_setting' );
				$this->bookingpress_merchant_email  = $BookingPress->bookingpress_get_settings( 'paypal_merchant_email', 'payment_setting' );
				$this->bookingpress_api_username    = $BookingPress->bookingpress_get_settings( 'paypal_api_username', 'payment_setting' );
				$this->bookingpress_api_password    = $BookingPress->bookingpress_get_settings( 'paypal_api_password', 'payment_setting' );
				$this->bookingpress_api_signature   = $BookingPress->bookingpress_get_settings( 'paypal_api_signature', 'payment_setting' );

				$PayPalConfig = array(
					'Sandbox'      => $this->bookingpress_is_sandbox_mode,
					'APIUsername'  => $this->bookingpress_api_username,
					'APIPassword'  => $this->bookingpress_api_password,
					'APISignature' => $this->bookingpress_api_signature,
				);

				$PayPal = new PayPal( $PayPalConfig );
			}

			return $PayPal;
		}


		function bookingpress_submit_form_data( $return_response, $bookingpress_return_data ) {
			if ( ! empty( $bookingpress_return_data ) ) {
				global $BookingPress,$bookingpress_debug_payment_log_id;

				$paypal_data = $this->arm_init_paypal();

				$entry_id                          = $bookingpress_return_data['entry_id'];
				$currency                          = $bookingpress_return_data['currency'];
				$currency_symbol                   = $BookingPress->bookingpress_get_currency_symbol( $currency );
				$bookingpress_final_payable_amount = isset( $bookingpress_return_data['payable_amount'] ) ? $bookingpress_return_data['payable_amount'] : 0;
				$customer_details                  = $bookingpress_return_data['customer_details'];
				$customer_email                    = ! empty( $customer_details['customer_email'] ) ? $customer_details['customer_email'] : '';

				$bookingpress_service_name = ! empty( $bookingpress_return_data['service_data']['bookingpress_service_name'] ) ? $bookingpress_return_data['service_data']['bookingpress_service_name'] : __( 'Appointment Booking', 'bookingpress-appointment-booking' );

				$custom_var = $entry_id;

				$sandbox = $this->bookingpress_is_sandbox_mode ? 'sandbox.' : '';

				$notify_url = $bookingpress_return_data['notify_url'];

				$redirect_url                    = $bookingpress_return_data['approved_appointment_url'] . '?appointment_id=' . base64_encode( $entry_id );
				$bookingpress_appointment_status = $BookingPress->bookingpress_get_settings( 'appointment_status', 'general_setting' );
				if ( $bookingpress_appointment_status == 'Pending' ) {
					$redirect_url = $bookingpress_return_data['pending_appointment_url'] . '?appointment_id=' . base64_encode( $entry_id );
				}

				$bookingpress_paypal_cancel_url = $BookingPress->bookingpress_get_settings( 'paypal_cancel_url', 'payment_setting' );				
				$cancel_url = !empty($bookingpress_paypal_cancel_url) ? $bookingpress_paypal_cancel_url : BOOKINGPRESS_HOME_URL ;				
				$cancel_url = add_query_arg( 'is_cancel', 1 , esc_url( $cancel_url ) );

				$cmd          = '_xclick';
				$paypal_form  = '<form name="_xclick" id="bookingpress_paypal_form" action="https://www.' . $sandbox . 'paypal.com/cgi-bin/webscr" method="post">';
				$paypal_form .= '<input type="hidden" name="cmd" value="' . $cmd . '" />';
				$paypal_form .= '<input type="hidden" name="amount" value="' . $bookingpress_final_payable_amount . '" />';
				$paypal_form .= '<input type="hidden" name="business" value="' . $this->bookingpress_merchant_email . '" />';
				$paypal_form .= '<input type="hidden" name="notify_url" value="' . $notify_url . '" />';
				$paypal_form .= '<input type="hidden" name="cancel_return" value="' . $cancel_url . '" />';
				$paypal_form .= '<input type="hidden" name="return" value="' . $redirect_url . '" />';
				$paypal_form .= '<input type="hidden" name="rm" value="2" />';
				$paypal_form .= '<input type="hidden" name="lc" value="en_US" />';
				$paypal_form .= '<input type="hidden" name="no_shipping" value="1" />';
				$paypal_form .= '<input type="hidden" name="custom" value="' . $custom_var . '" />';
				$paypal_form .= '<input type="hidden" name="on0" value="user_email" />';
				$paypal_form .= '<input type="hidden" name="os0" value="' . $customer_email . '" />';
				$paypal_form .= '<input type="hidden" name="page_style" value="primary" />';
				$paypal_form .= '<input type="hidden" name="charset" value="UTF-8" />';
				$paypal_form .= '<input type="hidden" name="item_name" value="' . $bookingpress_service_name . '" />';
				$paypal_form .= '<input type="hidden" name="item_number" value="1" />';
				$paypal_form .= '<input type="submit" value="Pay with PayPal!" />';
				$paypal_form .= '</form>';

				do_action( 'bookingpress_payment_log_entry', 'paypal', 'payment form redirected data', 'bookingpress', $paypal_form, $bookingpress_debug_payment_log_id );

				$paypal_form .= '<script type="text/javascript">document.getElementById("bookingpress_paypal_form").submit();</script>';

				$return_response['variant']       = 'redirect';
				$return_response['title']         = '';
				$return_response['msg']           = '';
				$return_response['is_redirect']   = 1;
				$return_response['redirect_data'] = $paypal_form;
				$return_response['entry_id'] = $entry_id;					
			}
			return $return_response;
		}

		function bookingpress_payment_gateway_data() {
			global $wpdb, $BookingPress, $bookingpress_payment_gateways, $bookingpress_debug_payment_log_id;
			if ( ! empty( $_REQUEST['bookingpress-listener'] ) ) {
				$bookingpress_webhook_data = $_REQUEST;
				do_action( 'bookingpress_payment_log_entry', 'paypal', 'Paypal Webhook Data', 'bookingpress', $bookingpress_webhook_data, $bookingpress_debug_payment_log_id );
				if ( ! empty( $bookingpress_webhook_data ) && ! empty( $bookingpress_webhook_data['custom'] ) ) {
					$entry_id       = intval( $bookingpress_webhook_data['custom'] );
					$payment_status = ! empty( $bookingpress_webhook_data['payment_status'] ) ? sanitize_text_field( $bookingpress_webhook_data['payment_status'] ) : 'success';
					$payer_email    = ! empty( $bookingpress_webhook_data['payer_email'] ) ? sanitize_email( $bookingpress_webhook_data['payer_email'] ) : '';
					$bookingpress_webhook_data['bookingpress_payer_email'] = $payer_email;
					$bookingpress_webhook_data                             = array_map( array( $BookingPress, 'appointment_sanatize_field' ), $bookingpress_webhook_data );
					$payment_log_id                                        = $bookingpress_payment_gateways->bookingpress_confirm_booking( $entry_id, $bookingpress_webhook_data, $payment_status, 'txn_id' );
				}
			}
		}
	}
}

global $bookingpress_class_paypal;
$bookingpress_class_paypal = new bookingpress_class_paypal();
